Package: mbkmeans
Type: Package
Title: Mini-batch K-means Clustering for Single-Cell RNA-seq
Version: 1.28.0
Authors@R: 
    c(person("Yuwei", "Ni", role = c("aut", "cph"),
             email = "yuweini45@gmail.com"),
      person("Davide", "Risso", role = c("aut", "cre", "cph"),
             email = "risso.davide@gmail.com"),
      person("Stephanie", "Hicks", role = c("aut", "cph"),
             email = "shicks19@jhu.edu"),
      person("Elizabeth", "Purdom", role = c("aut", "cph"),
             email = "epurdom@stat.berkeley.edu"))
Description: 
    Implements the mini-batch k-means algorithm for large
    datasets, including support for on-disk data representation.
Depends: R (>= 3.6)
Imports: methods, DelayedArray, Rcpp, S4Vectors, SingleCellExperiment,
        SummarizedExperiment, ClusterR, benchmarkme, Matrix,
        BiocParallel
Suggests: beachmat, HDF5Array, Rhdf5lib, BiocStyle, TENxPBMCData,
        scater, DelayedMatrixStats, bluster, knitr, testthat, rmarkdown
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
LinkingTo: Rcpp, RcppArmadillo (>= 0.7.2), Rhdf5lib, beachmat, ClusterR
SystemRequirements: C++11
VignetteBuilder: knitr
biocViews: Clustering, GeneExpression, RNASeq, Software,
        Transcriptomics, Sequencing, SingleCell
BugReports: https://github.com/drisso/mbkmeans/issues
git_url: https://git.bioconductor.org/packages/mbkmeans
git_branch: RELEASE_3_23
git_last_commit: a415e0d
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 23:04:51 UTC; biocbuild
Author: Yuwei Ni [aut, cph],
  Davide Risso [aut, cre, cph],
  Stephanie Hicks [aut, cph],
  Elizabeth Purdom [aut, cph]
Maintainer: Davide Risso <risso.davide@gmail.com>
