# Generated by roxygen2: do not edit by hand

export(MbkmeansParam)
export(blocksize)
export(compute_wcss)
export(mini_batch)
export(predict_mini_batch)
export(predict_mini_batch_r)
exportMethods(mbkmeans)
import(ClusterR)
importClassesFrom(DelayedArray,DelayedMatrix)
importClassesFrom(SingleCellExperiment,LinearEmbeddingMatrix)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(BiocParallel,SerialParam)
importFrom(DelayedArray,blockApply)
importFrom(DelayedArray,colAutoGrid)
importFrom(DelayedArray,colSums)
importFrom(Matrix,t)
importFrom(Rcpp,sourceCpp)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SingleCellExperiment,sampleFactors)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(benchmarkme,get_ram)
importFrom(methods,is)
useDynLib(mbkmeans, .registration = TRUE)
