#' @rdname mbkmeans
setGeneric(
    name = "mbkmeans",
    def = function(x, ...) standardGeneric("mbkmeans")
)

#' Cluster rows of a matrix
#'
#' Cluster rows of a matrix-like object with a variety of algorithms.
#'
#' This function is deprecated. Please use the \code{clusterRows} function in
#' the \code{bluster} Bioconductor package.
#' @name clusterRows
NULL